from typing import Dict, List

from ..type_specs import HotkeysWithModes
from ._base import Parser
from .sources import Command, PythonModule


class Mpv(Parser):
    required = all([Command("mpv"), PythonModule("mpv")])
    has_modes = True

    def fetch(self) -> List[Dict[str, str]]:
        import mpv

        player = mpv.MPV()  # type: ignore
        out = player.input_bindings
        player.terminate()
        return out

    def parse(self, fetched: List[Dict[str, str]]) -> HotkeysWithModes:
        out = {}
        for binding in fetched:
            if binding["section"] not in out:
                out[binding["section"]] = {}
            out[binding["section"]][binding["key"]] = binding.get(
                "comment", binding.get("cmd")
            )
        return out



import os
from pathlib import Path
from typing import Dict, List

from ..type_specs import Hotkeys
from ._base import Parser
from .sources import Command, File


class Sioyek(Parser):
    required = bool(Command("sioyek"))
    sources = {
        "system": [File(Path("/etc/sioyek/keys.config"))],
        "user": [
            File(Path(os.environ["XDG_CONFIG_HOME"]) / "sioyek" / "keys_user.config")
        ],
    }
    has_modes = False

    def parse(self, fetched: Dict[str, List[str]]) -> Hotkeys:
        out = {}
        for _, contents in fetched.items():
            for content in contents:
                content = content.split("\n")
                for line in content:
                    if line.startswith("#") or len(line.strip()) == 0:
                        continue
                    (command, key) = line.split()
                    out[key] = command
        return out



import os
from pathlib import Path
from typing import Dict, List

from ..type_specs import HotkeysWithModes
from ._base import Parser
from .sources import Command, File, PythonModule, Web


class Alacritty(Parser):
    required = all([Command("alacritty"), PythonModule("yaml")])
    sources = {
        "default": [File(Path("/usr/share/doc/alacritty/example/alacritty.yml"))],
        "default_web": [
            Web(
                "https://raw.githubusercontent.com/alacritty/alacritty/master/alacritty.yml"
            ),
        ],
        "user": [
            File(
                Path(
                    os.environ.get(
                        "XDG_CONFIG_HOME",
                        Path(os.environ["HOME"]) / ".config",
                    )
                )
                / "alacritty"
                / "alacritty.yml"
            )
        ],
    }
    has_modes = True

    @staticmethod
    def _format_key(bind) -> str:
        if "mods" in bind:
            return f"{bind.get('mods', '').replace('|', '+')}+{bind['key']}"
        else:
            return bind["key"]

    @staticmethod
    def _clean_default(contents: str) -> str:
        keep_line = False
        out = []
        for line in contents.split("\n"):
            if keep_line:
                out.append(line.replace("#-", "-"))
            if line.startswith("#debug"):
                keep_line = False
            if line.startswith("#key_bindings"):
                keep_line = True
                out.append(line.replace("#", ""))
        return "\n".join(out)

    def parse(self, fetched: Dict[str, List[str]]) -> HotkeysWithModes:
        import yaml

        out = {}
        for source, contents in fetched.items():
            contents = contents[0]
            if source in ["default_web", "default"]:
                contents = self._clean_default(contents)
            config_yml = yaml.safe_load(contents)
            for bind in config_yml["key_bindings"]:
                mode = bind.get("mode", "normal")
                if mode not in out:
                    out[mode] = {}
                out[mode][self._format_key(bind)] = bind.get(
                    "action", bind.get("chars")
                )
        return out



from typing import Dict, List

from ..type_specs import HotkeysWithModes
from ._base import Parser
from .sources import Command, PythonModule
from .vim import MODE_MAP


class Neovim(Parser):
    required = all([Command("nvim"), PythonModule("pynvim")])
    has_modes = True

    def fetch(self) -> List[Dict[str, str]]:
        import pynvim

        nvim = pynvim.attach(
            "child", argv=["/bin/env", "nvim", "--embed", "--headless"]
        )
        out = nvim.api.get_keymap("*")
        nvim.quit()
        nvim.close()
        return out

    def parse(self, fetched: List[Dict[str, str]]) -> HotkeysWithModes:
        out = {}
        for map in fetched:
            mode = MODE_MAP[map["mode"]]
            if mode not in out:
                out[mode] = {}
            if map["lhs"].startswith("<Plug>") or not any(
                ["desc" in map, "rhs" in map]
            ):
                continue
            out[mode][map["lhs"].replace(" ", "<space>")] = map.get(
                "desc", map.get("rhs")
            )
        return out



import re

from ..type_specs import HotkeysWithModes
from ._base import Parser
from .sources import Command

MODE_MAP = {
    " ": "Normal, Visual, Select, Operator-pending",
    "n": "Normal",
    "v": "Visual, Select",
    "s": "Select",
    "x": "Visual",
    "o": "Operator-pending",
    "!": "Insert, Command-line",
    "i": "Insert",
    "l": '":lmap" mappings for Insert, Command-line and Lang-Arg',
    "c": "Command-line",
    "t": "Terminal-Job",
}


class Vim(Parser):
    required = bool(Command("vim"))
    # map won't show autocmd added hotkeys
    sources = {
        "user": [
            Command('vim -e +"redir >> /dev/stdout | map | redir END" -scq'),
        ]
    }
    has_modes = True

    def parse(self, fetched) -> HotkeysWithModes:
        # convert vim mode notation to human, from :help map
        content = fetched["user"][0]
        # get the key/action
        content_key_action = re.compile(r"(.)\s+(\S+)\s+[@\&\*]?\s*(.*?)\n")
        out = {}
        for match in re.finditer(content_key_action, content):
            key = match[2]
            # remove <Plug> keys
            if not key.startswith("<Plug>"):
                mode = MODE_MAP[match[1]]
                action = match[3]
                if mode not in out.keys():
                    out[mode] = {}
                out[mode][key] = action
        return out



from ..type_specs import HotkeysWithModes
from ._base import Parser
from .sources import PythonModule


class Vimiv(Parser):
    required = bool(PythonModule("vimiv"))
    has_modes = True

    def fetch(self) -> HotkeysWithModes:
        from vimiv import api

        return {mode: dict(keys) for mode, keys in api.keybindings.items()}  # type: ignore

    def parse(self, fetched: HotkeysWithModes) -> HotkeysWithModes:
        return fetched
